#!/usr/bin/perl

# RelativeLayers 0.9.7
# 
# $Id: stripjavascriptcode.pl 1.8 04/02/02 21:22:32+01:00 gbevin@willow.uwyn.office $
# 
# Library for creating dynamic designs that adapt themselves to the user's
# browser environment.
# 
# Copyright (C) 1998-2004, Geert Bevin
# Distributed under the terms of the GNU Lesser General Public, v2.1 or later
# 
# gbevin[remove] at uwyn dot com
# http://www.uwyn.com/projects/relativelayers

my(@filesToProcess) = @ARGV;
if(scalar @filesToProcess == 0)
{
    @filesToProcess = ("rl_layer.js", "rl_window.js", "rl_utility.js", "rl_dragdrop.js", "rl_limits.js", "rl_source.js", "rl_browser.js", "rl_debug.js", "rl_error.js");
}

%functionsToSkip =
(
	"unsupportedBrowser" => 1,
	
    "Hashtable" => 1,
	"showProperties" => 1,

	"startDebug" => 1,
	"stopDebug" => 1,
	"debug" => 1,
	
	"addLoadHook" => 1,
	"addResizeHook" => 1,
	"getWindowWidth" => 1,
	"getWindowHeight" => 1,
	"nextHook" => 1,
	
	"getRelativeLayer" => 1,
	"RelativeLayer" => 1,
	"normalizeEvent" => 1,

	"finishContentsUpdate" => 1
);

%methodsToSkip =
(
    "contains" => 1,
    "indexOf" => 1,
    "pop" => 1,
    "push" => 1,
    "remove" => 1,
    
    "clear" => 1,
    "containsKey" => 1,
    "containsValue" => 1,
    "get" => 1,
    "keys" => 1,
    "put" => 1,
    "remove" => 1,
    "size" => 1,
    "values" => 1,
	
    "chop" => 1,
	
	"addEventListener" => 1,
	"availableMoveBy" => 1,
	"availableMoveTo" => 1,
	"availableResizeBy" => 1,
	"availableResizeTo" => 1,
	"calculateAvailableHandleHorizOffset" => 1,
	"calculateAvailableHandleVertOffset" => 1,
	"calculateAvailableHeight" => 1,
	"calculateAvailableHorizOffset" => 1,
	"calculateAvailableVertOffset" => 1,
	"calculateAvailableWidth" => 1,
	"calculateVisibleHandleHorizOffset" => 1,
	"calculateVisibleHandleVertOffset" => 1,
	"calculateVisibleHeight" => 1,
	"calculateVisibleHorizOffset" => 1,
	"calculateVisibleVertOffset" => 1,
	"calculateVisibleWidth" => 1,
	"createLayer" => 1,
	"destroyLayer" => 1,
	"fireEventObject" => 1,
	"fireEvent" => 1,
	"getAbsoluteHeight" => 1,
	"getAbsolutePageX" => 1,
	"getAbsolutePageY" => 1,
	"getAbsoluteWidth" => 1,
	"getAbsoluteX" => 1,
	"getAbsoluteY" => 1,
	"getAvailableHandleHorizOffset" => 1,
	"getAvailableHandleVertOffset" => 1,
	"getAvailableHeight" => 1,
	"getAvailableHorizBorder" => 1,
	"getAvailableHorizOffset" => 1,
	"getAvailableVertBorder" => 1,
	"getAvailableVertOffset" => 1,
	"getAvailableWidth" => 1,
	"getBgColor" => 1,
	"getBgImage" => 1,
	"getId" => 1,
	"getParentLayer" => 1,
	"getVisibleHandleHorizOffset" => 1,
	"getVisibleHandleVertOffset" => 1,
	"getVisibleHeight" => 1,
	"getVisibleHorizBorder" => 1,
	"getVisibleHorizOffset" => 1,
	"getVisibleHorizOffsetReferenceId" => 1,
	"getVisibleVertBorder" => 1,
	"getVisibleVertOffset" => 1,
	"getVisibleVertOffsetReferenceId" => 1,
	"getVisibleWidth" => 1,
	"getZIndex" => 1,
	"intersects" => 1,
	"isVisible" => 1,
	"makeEventObject" => 1,
	"position" => 1,
	"raise" => 1,
	"refresh" => 1,
    "removeEventListener" => 1,
    "setAutoReposition" => 1,
	"setBgColor" => 1,
	"setBgImage" => 1,
	"setHtml" => 1,
	"setVisible" => 1,
	"setZIndex" => 1,
	"startScroll" => 1,
	"stopScroll" => 1,
	"visibleMoveBy" => 1,
	"visibleMoveTo" => 1,
	"visibleResizeBy" => 1,
	"visibleResizeTo" => 1,
	
	"addLimit" => 1,
	"addDimensionLimit" => 1,
	"clearLimits" => 1,
	
	"addDropTarget" => 1,
	"removeDropTarget" => 1,
	"setDraggable" => 1,
	"setDragLimit" => 1,
	"setDragAutoRaise" => 1,
	
	"setSource" => 1
);

my(%fileContents);

my($globalvarsNumber) = 0;
my(%replacedGlobalvars);
my($functionNumber) = 0;
my(%replacedFunctions);
my($methodNumber) = 0;
my(%replacedMethods);
my($memberNumber) = 0;
my(%replacedMembers);
my(%extensionFunctions);
my(%replacedVariables);
my(%replacedArguments);

print("FIRST PASS : Reading files ...\n");
foreach $file (@filesToProcess)
{
    print(". reading \"$file\"\n");
	my($filecontent);
	if(!open(FILE,"<$file"))
	{
		print("ERROR : Can't open file \"$file\".\nExiting ...\n");
        exit;
	}
	else
	{
		read(FILE, $filecontent, (stat(FILE))[7]);
		close(FILE);

        $fileContents{$file} = $filecontent;
        print(". successful\n");
    }
}

print("SECOND PASS : Removing comments, detecting global functions and the RelativeLayer object members and methods ...\n");
foreach $file (@filesToProcess)
{
    print(". processing \"$file\"\n");
	my($filecontent) = \$fileContents{$file};

    # remove comments
    while($$filecontent =~ s/\s*\/\/.*\n/ /g) {};
    print(".. removed comments\n");

	# get the global variables
    my(@globalvarsDup) = $$filecontent =~ m/\b(glb_\w+)\b/g;
    my($globalvars);
    my(%globalvars);
    # cancel duplicate global variables
    foreach $globalvars (@globalvarsDup)
    {
        $globalvars{$globalvars} = 1;
    }
    my($globalvarshort);
    foreach $globalvar (keys %globalvars)
    {
        if(length($globalvar) <= 3)
        {
            print(".. found global variable : \"$globalvar\" ... but it will not be replaced since it's not long enough\n");
        }
        elsif($replacedGlobalvars{$globalvar})
        {
            print(".. found global variable : \"$globalvar\" ... replacement has already been set to \"$replacedGlobalvars{$globalvar}\"\n");
        }
        else
        {
            $globalvarshort = "g".createID($globalvarNumber);
            $replacedGlobalvars{$globalvar} = $globalvarshort;
            print(".. found global variable : \"$globalvar\" ... replacement will be \"$globalvarshort\"\n");
            $globalvarNumber++;
        }
    }
    
    # get functions
    my(@functions) = $$filecontent =~ m/function\s+(\w+)\b/g;
    my($function);
    my($functionshort);
    foreach $function (@functions)
    {
        if(length($function) <= 3)
        {
            print(".. found function : \"$function\" ... but it will not be replaced since it's not long enough\n");
        }
        elsif($functionsToSkip{$function} == 1)
        {
            print(".. found function : \"$function\" ... but it's listed to be skipped\n");
        }
        elsif($replacedFunctions{$function})
        {
            print(".. found function : \"$function\" ... replacement has already been set to \"$functionsToSkip{$function}\"\n");
        }
        else
        {
            $functionshort = "f".createID($functionNumber);
            $replacedFunctions{$function} = $functionshort;
            print(".. found function : \"$function\" ... replacement will be \"$functionshort\"\n");
            $functionNumber++;
        }
    }
    
    # get methods that correspond to functions
    my(%methods);
    my($method);
    foreach $function (@functions)
    {
        # get corresponding method
        if($$filecontent =~ m/this\.(\w+)\s*=\s*$function\s*\;/)
        {
            $method = $+;
            $methods{$method} = 1;
        }
    }
    # get dynamic declared functions
    my(@dynmethods) = $$filecontent =~ m/\.prototype\.(\w+)\b/g;
    my($dynmethod);
    foreach $dynmethod (@dynmethods)
    {
        $methods{$dynmethod} = 1;
    }
    
    # create replacements for the methods
    my($methodshort);
    foreach $method (keys %methods)
    {
        if(length($method) <= 3)
        {
            print(".. found method : \"$method\" ... but it will not be replaced since it's not long enough\n");
        }
        elsif($methodsToSkip{$method} == 1)
        {
            print(".. found method : \"$method\" ... but it's listed to be skipped\n");
        }
        elsif($replacedMethods{$method})
        {
            print(".. found method : \"$method\" ... replacement has already been set to \"$replacedMethods{$method}\"\n");
        }
        else
        {
            $methodshort = "m".createID($methodNumber);
            $replacedMethods{$method} = $methodshort;
            print("... found method : \"$method\" ... replacement will be \"$methodshort\"\n");
            $methodNumber++;
        }
    }

    # detect the extension functions
    my(@extensions) = $$filecontent =~ m/extendMethod\([^,]+\,\s*RelativeLayer\.prototype\.([^,\s]+)\s*,\s*([^,)\s]+)\s*/g;
    my($index);
    while($index <= $#extensions)
    {
        $extensionFunctions{$extensions[$index+1]} = $extensions[$index];
        print(".. found extension : \"$extensions[$index+1]\" of \"$extensions[$index]\"\n");
        $index += 2;
    }

    # get the members
    my(@membersDup) = $$filecontent =~ m/\b(mbr_\w+)\b/g;
    my($member);
    my(%members);
    # cancel duplicate members
    foreach $member (@membersDup)
    {
        $members{$member} = 1;
    }
    my($membershort);
    foreach $member (keys %members)
    {
        if(length($member) <= 3)
        {
            print(".. found member : \"$member\" ... but it will not be replaced since it's not long enough\n");
        }
        elsif($replacedMembers{$member})
        {
            print(".. found member : \"$member\" ... replacement has already been set to \"$replacedMembers{$member}\"\n");
        }
        else
        {
            $membershort = "n".createID($memberNumber);
            $replacedMembers{$member} = $membershort;
            print(".. found member : \"$member\" ... replacement will be \"$membershort\"\n");
            $memberNumber++;
        }
    }
    
    print(". successful\n");
}

print("THIRD PASS : Replace the arguments and local variables inside functions ...\n");
foreach $file (@filesToProcess)
{
    print(". processing \"$file\"\n");
	my($filecontent) = \$fileContents{$file};
    
    # obtain the names of the functions
    my(@functiondefinitionparts) = $$filecontent =~ m/(RelativeLayer\.prototype\.)?(\w*)\s*=?\s*[\s\n\r\t]*\bfunction\b\s*(\w*)\b/g;
    my(@functionnames);
    my($index);
    while($index <= $#functiondefinitionparts)
    {
        if($functiondefinitionparts[$index+1])
        {
            push(@functionnames, $functiondefinitionparts[$index+1]);
        }
        elsif($functiondefinitionparts[$index+2])
        {
            push(@functionnames, $functiondefinitionparts[$index+2]);
        }
        $index += 3;
    }
    print(".. detected ".scalar(@functionnames)." function names and stored them\n");
    
    # split the code in parts according to the functions
    my(@codeparts) = split(/[\s\n\r\t]*\bfunction[\s\t\r\t\(]+/, $$filecontent);
    print(".. splitted the file into ".scalar(@codeparts)." code parts according to the function definitions\n");

    # check if the number of detected functions corresponds to the number of parts
    if($#codeparts-1 != $#functionnames)
    {
        print("ERROR : there should be one function name less than there are parts.\nExiting ...\n");
        exit;
    }
    
    # process every code parts individually
    my($functionnumber) = 1;
    while($functionnumber <= $#codeparts)
    {
        my($functionname) = $functionnames[$functionnumber-1]; 
        if(length($codeparts[$functionnumber-1]) &&
           substr($codeparts[$functionnumber-1], length($codeparts[$functionnumber-1])-1, 1) eq "=")
        {
            $codeparts[$functionnumber] = 'function('.$codeparts[$functionnumber];
        }
        else
        {
            $codeparts[$functionnumber] = 'function '.$codeparts[$functionnumber];
        }

        if(exists($extensionFunctions{$functionname}))
        {
            print("... skipping function \"$functionname\" since it's an extension function\n");
        }
        else
        {
            print("... processing function \"$functionname\"\n");
            # get functionarguments
            my(@functionarguments) = $codeparts[$functionnumber] =~ m/function\s*\w*\s*\(([\w\,\s\n\r\t]*)\)/g; 
            my(%arguments);
            foreach $key (@functionarguments)
            {
                if($key =~ /^par_/)
                {
                    $key =~ s/[\s\n\t\r]*//;
                    my(@functionargumentsparts) = split(/\s*,\s*/,$key);
                    foreach $functionargumentspart (@functionargumentsparts)
                    {
                        if($functionargumentspart)
                        {
                            $arguments{"$functionargumentspart"} = 1;
                        }
                    }
                }
            }

            # replace function arguments
            my($argumentnumber) = 0;
            my($argumentshort);
            foreach $argument (keys %arguments)
            {
                if(length($argument)>2)
                {
                    $argumentshort = "a".createID($argumentnumber);
                    $codeparts[$functionnumber] =~ s/\b$argument\b/$argumentshort/g;
                    $replacedArguments{$functionname}{$argument} = $argumentshort;
                    print ".... replaced argument \"$argument\" with \"$argumentshort\"\n";
                    $argumentnumber++;
                }
            }

            # get local variables
            my(@localvariables) = $codeparts[$functionnumber] =~ m/\b(var_\w+)\b/g;
            my(%variables);
            foreach $key (@localvariables)
            {
                $variables{$key} = 1;
            }

            # replace local variables
            my($variablenumber) = 0;
            my($variableshort);
            foreach $variable (keys %variables)
            {
                if(length($variable)>2)
                {
                    $variableshort = "v".createID($variablenumber);
                    $codeparts[$functionnumber] =~ s/\b$variable\b/$variableshort/g;
                    $replacedVariables{$functionname}{$variable} = $variableshort;
                    print ".... replaced local variable \"$variable\" with \"$variableshort\"\n";
                    $variablenumber++;
                }
            }
        }

        $functionnumber++;
    }

    $$filecontent = join(//,@codeparts);
    print(".. glued code parts back together\n");
}

print("FOURTH PASS : Replace the arguments and local variables inside functions extensions...\n");
foreach $file (@filesToProcess)
{
    print(". processing \"$file\"\n");
	my($filecontent) = \$fileContents{$file};
    
    # obtain the names of the functions
    my(@functiondefinitionparts) = $$filecontent =~ m/(RelativeLayer\.prototype\.)?(\w*)\s*=?\s*[\s\n\r\t]*\bfunction\b\s*(\w*)\b/g;
    my(@functionnames);
    my($index);
    while($index <= $#functiondefinitionparts)
    {
        if($functiondefinitionparts[$index+1])
        {
            push(@functionnames, $functiondefinitionparts[$index+1]);
        }
        elsif($functiondefinitionparts[$index+2])
        {
            push(@functionnames, $functiondefinitionparts[$index+2]);
        }
        $index += 3;
    }
    print(".. detected ".scalar(@functionnames)." function names and stored them\n");
    
    # split the code in parts according to the functions
    my(@codeparts) = split(/[\s\n\r\t]*\bfunction[\s\t\r\t\(]+/, $$filecontent);
    print(".. splitted the file into ".scalar(@codeparts)." code parts according to the function definitions\n");

    # check if the number of detected functions corresponds to the number of parts
    if($#codeparts-1 != $#functionnames)
    {
        print("ERROR : there should be one function name less than there are parts.\nExiting ...\n");
        exit;
    }
    
    # process every code parts individually
    my($functionnumber) = 1;
    while($functionnumber <= $#codeparts)
    {
        my($functionname) = $functionnames[$functionnumber-1]; 
        if(length($codeparts[$functionnumber-1]) &&
           substr($codeparts[$functionnumber-1], length($codeparts[$functionnumber-1])-1, 1) eq "=")
        {
            $codeparts[$functionnumber] = 'function('.$codeparts[$functionnumber];
        }
        else
        {
            $codeparts[$functionnumber] = 'function '.$codeparts[$functionnumber];
        }

        if(exists($extensionFunctions{$functionname}))
        {
            print("... processing extension function \"$functionname\" which extends \"$extensionFunctions{$functionname}\"\n");
            
            # get functionarguments
            my(@functionarguments) = $codeparts[$functionnumber] =~ m/\b(par_\w+)\b/g;
            my(%arguments);
            foreach $key (@functionarguments)
            {
                $arguments{$key} = 1;
            }

            # replace function arguments
            foreach $argument (keys %arguments)
            {
                if(length($argument)>2)
                {
                    if(exists($replacedArguments{$extensionFunctions{$functionname}}))
                    {
                        my(%existing_replacements) = %{$replacedArguments{$extensionFunctions{$functionname}}};

                        if(exists($existing_replacements{$argument}))
                        {
                            my($argumentshort) = $existing_replacements{$argument};
                            
                            $codeparts[$functionnumber] =~ s/\b$argument\b/$argumentshort/g;
                            print ".... replaced argument \"$argument\" with \"$argumentshort\"\n";
                        }
                    }
                }
            }

            # get local variables
            my(@localvariables) = $codeparts[$functionnumber] =~ m/\b(var_\w+)\b/g;
            my(%variables);
            foreach $key (@localvariables)
            {
                $variables{$key} = 1;
            }

            # replace local variables
            my($variablenumber) = 0;
            my($variableshort);
            foreach $variable (keys %variables)
            {
                if(length($variable)>2)
                {
                    $variableshort = "d".createID($variablenumber);
                    if(exists($replacedVariables{$extensionFunctions{$functionname}}))
                    {
                        my(%existing_replacements) = %{$replacedVariables{$extensionFunctions{$functionname}}};

                        if(exists($existing_replacements{$variable}))
                        {
                            $variableshort = $existing_replacements{$variable};
                        }
                    }
                    $codeparts[$functionnumber] =~ s/\b$variable\b/$variableshort/g;
                    print ".... replaced local variable \"$variable\" with \"$variableshort\"\n";
                    $variablenumber++;
                }
            }
        }

        $functionnumber++;
    }

    $$filecontent = join(//,@codeparts);
    print(".. glued code parts back together\n");
}

print("FIFTH PASS : Modifying the file's content to compact it ...\n");
foreach $file (@filesToProcess)
{
    print(". processing \"$file\"\n");
	my($filecontent) = \$fileContents{$file};
    
    # remove newlines
    $$filecontent =~ s/[\s\t]*[\n\r]+/ /g;
    print(".. removed newlines\n");

    # remove tabs
    $$filecontent =~ s/\t+/ /g;
    print(".. removed tabs\n");
    
    # remove multiple spaces
    $$filecontent =~ s/\s+/ /g;
    print(".. removed multiple spaces\n");

    # remove spaces before and behind certain operators
    $$filecontent =~ s/\s*([\{\}\(\)\=\!\<\>\,\;\+\-\\\*])\s*/\1/g;
    print(".. remove spaces before and behind {}()=!<>,;+-\\*\n");

    # remove spaces at the beginning
    $$filecontent =~ s/^\s+//g;
    print(".. removed spaces at the beginning\n");

    # remove spaces at the end
    $$filecontent =~ s/\s+$//g;
    print(".. removed spaces at the end\n");

    # replace the global variables by their shorter names
    my($globalvar);
    foreach $globalvar (keys %replacedGlobalvars)
    {
        if($$filecontent =~ m/\b$globalvar\b/g)
        {
            $$filecontent =~ s/\b$globalvar\b/$replacedGlobalvars{$globalvar}/g;
            print(".. replaced global variable \"$globalvar\" with \"$replacedGlobalvars{$globalvar}\".\n");
        }
    }
        
    # replace the members by their shorter names
    my($member);
    foreach $member (keys %replacedMembers)
    {
        if($$filecontent =~ m/\b$member\b/g)
        {
            $$filecontent =~ s/\b$member\b/$replacedMembers{$member}/g;
            print(".. replaced member \"$member\" with \"$replacedMembers{$member}\".\n");
        }
    }
        
    # replace the functions by their shorter names
    my($function);
    foreach $function (keys %replacedFunctions)
    {
        if($$filecontent =~ m/\b$function\b/gs)
        {
            $$filecontent =~ s/\b$function\b/$replacedFunctions{$function}/gs;
            print(".. replaced function \"$function\" with \"$replacedFunctions{$function}\"\n");
        }
    }

    # replace the methods by their shorter names
    my($method);
    foreach $method (keys %replacedMethods)
    {
        if($$filecontent =~ m/\.$method\b/gs ||
           $$filecontent =~ m/\.prototype\.$method\b/gs)
        {
            $$filecontent =~ s/\.$method\b/\.$replacedMethods{$method}/gs;
            $$filecontent =~ s/\.prototype\.$method\b/\.prototype\.$replacedMethods{$method}/gs;
            print(".. replaced method \"$method\" with \"$replacedMethods{$method}\"\n");
        }
    }

    print(". successful\n");
}

print("SIXTH PASS : Saving the modified files ...\n");
foreach $file (@filesToProcess)
{
	my($filecontent) = \$fileContents{$file};
    
    my($filename, $fileextension) = split(/\./,$file);
    my($filenametarget) = "$filename\_stripped.js";
    if(!open(FILE,">$filenametarget"))
    {
        print("ERROR : Can't open file \"$filenametarget\" for writing.Exiting ...\n\n");
        exit;
    }
    else
    {
        print FILE  $$filecontent;
        close(FILE);
        print(". wrote file \"$file\" to \"$filenametarget\"\n");
    }
}
exit;

sub createID()
{
    my($number) = $_[0];
    my($result);

    my($division_result) = int($number / 62);
    my($remainder) = $number % 62;
	if($remainder < 26)
	{
		$remainder = chr($remainder+65);
	}
	elsif($remainder < 52)
	{
		$remainder = chr($remainder+71);
	}
	elsif($remainder < 62)
	{
		$remainder = chr($remainder-4);
	}
    if($division_result > 0)
    {
        $result = createID($division_result);
    }
    $result = $result.$remainder;

	return($result);
};
